/*
 * bmon/db.h		Database Layer
 *
 * Copyright (c) 2001-2005 Thomas Graf <tgraf@suug.ch>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __BMON_DB_H_
#define __BMON_DB_H_

#include <bmon/bmon.h>
#include <bmon/node.h>
#include <bmon/item.h>

#include <libpq-fe.h>

#define DB_NODELEN	32
#define DB_SOURCELEN	64
#define DB_TSLEN	32
#define DB_DESCLEN	64
#define DB_ITEMLEN	IFNAME_MAX
#define DB_ATTRLEN	16
#define DB_TXTLEN	32

struct db_node
{
	int	index;
	int	db_id;
	char	name[DB_NODELEN];
};

struct db_node_full
{
	int	index;
	int	db_id;
	char	name[DB_NODELEN];
	char	source[DB_SOURCELEN];
	char	created[DB_TSLEN];
	char	last_update[DB_TSLEN];
};

struct db_item
{
	int	index;
	int	db_id;
	int	handle;
	int	parent;
	int	node;
	char	name[DB_ITEMLEN];
};

struct db_cnt
{
	int	item_index;
	char	item_name[DB_ITEMLEN];
	int	item_handle;
	int	item_parent;
	int	item_indent;
	int	item_major_attr;
	int	item_minor_attr;
	char	attr_type[DB_ATTRLEN];
	b_cnt_t	attr_rx_cnt;
	b_cnt_t	attr_tx_cnt;
	char	last_update[DB_TSLEN];

	void *	arg; /* owned by users of db_cnt */
};

struct db_item_full
{
	int	index;
	int	db_id;
	int	handle;
	int	parent;
	int	node;
	int	rx_usage;
	int	tx_usage;
	int	indent;
	char	name[DB_ITEMLEN];
	char	desc[DB_DESCLEN];
	char	created[DB_TSLEN];
	char	last_update[DB_TSLEN];
};

struct db_attr
{
	int	index;
	int	db_id;
	char	name[DB_ATTRLEN];
	int	item;
};

struct db_attr_full
{
	int	index;
	int	db_id;
	char	name[DB_ATTRLEN];
	int	item;
	int	unit;
	int	rx_rate;
	int	tx_rate;
	b_cnt_t rx_cnt;
	b_cnt_t tx_cnt;
	char	hts[DB_TSLEN];
	char	created[DB_TSLEN];
	char	last_update[DB_TSLEN];
	char	txt[DB_TXTLEN];
};

struct db_rule_settings
{
	int	up_read;
	int	up_sec;
	int	up_min;
	int	up_hour;
	int	up_day;
	char *  expire_read;
	char *  expire_sec;
	char *  expire_min;
	char *  expire_hour;
	char *  expire_day;
};

struct db_collection
{
	int	size;
	void *	array;
};

#define HP_HAS_FROM	1
#define HP_HAS_TO	2

struct db_hist_params
{
	xdate_t 	hp_from;
	xdate_t 	hp_to;
	int		hp_flags;
};

static inline int has_params(struct db_hist_params *p)
{
	return p->hp_flags;
}

#define for_each_entry(pos, head, type) \
	for (pos = (head)->array; \
	     pos < ((type *) (head)->array + (head)->size); \
	     pos++)

static inline void bdb_destroy_collection(struct db_collection *col)
{
	free(col->array);
}

#define DB_T_WILDCARD	1
#define DB_T_ATTR	2

struct db_token
{
	int	t_id;
	char *  t_name;
	int     t_flags;

	struct db_token * t_next;
};

struct db_filter
{
	struct db_token	* f_node;
	struct db_token	* f_item;
	struct db_token	* f_attr;
	char *		  f_field;

	struct db_id *	  f_next;
};

extern PGconn* bdb_connect(const char *host, const char *user,
		       const char *password, const char *dbname);
extern void bdb_shutdown(PGconn *);

extern int  bdb_create_tables(PGconn *);
extern int  bdb_insert_attr_descs(PGconn *);
extern int  bdb_drop_tables(PGconn *);
extern int  bdb_reinstall_rules(PGconn *, struct db_rule_settings *);
extern int  bdb_drop_rules(PGconn *);

extern int  bdb_begin(PGconn *);
extern int  bdb_commit(PGconn *);
extern int  bdb_analyze(PGconn *);

extern int  bdb_update_caches(PGconn *);
extern struct db_node * bdb_get_node_cache(void);
extern int  bdb_get_node_cache_rows(void);
extern struct db_item * bdb_get_item_cache(void);
extern int  bdb_get_item_cache_rows(void);
extern struct db_attr * bdb_get_attr_cache(void);
extern int  bdb_get_attr_cache_rows(void);

extern int  bdb_get_node(const char *name);
extern int  bdb_find_node(struct node *node);
extern int  bdb_insert_node(PGconn *, struct node *node);

extern int  bdb_get_item(int node_id, const char *name, uint32_t handle,
			 int parent);
extern int  bdb_find_item(struct item *it, struct node *node, int node_id);
extern int  bdb_insert_item(PGconn *, struct item *it, struct node *node, int node_id);
extern int  bdb_update_item(PGconn *, struct item *it, int item_id);
extern int  bdb_finish_updates(PGconn *);

extern int  bdb_get_attr(const char *name, int item_id);
extern int  bdb_find_attr(struct stat_attr *attr, int item_id);
extern int  bdb_insert_attr(PGconn *, struct stat_attr *a, int item_id);
extern int  bdb_update_attr(PGconn *, struct stat_attr *a, int attr_id);

extern struct db_filter * bdb_parse_filter(const char *);

extern int  bdb_fetch_nodes(PGconn *, struct db_token *, struct db_collection *);
extern int  bdb_fetch_items(PGconn *, int, struct db_token *, struct db_collection *);
extern int  bdb_fetch_attrs(PGconn *, int, struct db_token *, struct db_collection *);
extern int  bdb_fetch_attr_hist(PGconn *, int, struct db_token *, struct db_collection *,
				struct db_hist_params *, const char *);

extern void bdb_print_filter(FILE *, struct db_filter *);

extern struct db_cnt * bdb_get_counters(PGconn *conn, int *len);

extern int bdb_parse_date(const char *str, xdate_t *dst);

#endif
