/*
 * bmon/item.h		Item Management
 *
 * Copyright (c) 2001-2004 Thomas Graf <tgraf@suug.ch>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __BMON_ITEM_H_
#define __BMON_ITEM_H_

#include <bmon/bmon.h>
#include <bmon/group.h>

#define OVERFLOW_LIMIT  4294967296ULL
#define MAX_GRAPHS 32
#define HISTORY_SIZE 60
#define IFNAME_MAX 32
#define ATTR_HASH_MAX 32
#define UNK_DATA UINT_MAX

struct hist_data
{
	b_cnt_t			hd_prev_total;
	rate_cnt_t		hd_data[HISTORY_SIZE];
};

struct hist_elem
{
	struct hist_data	he_rx;
	struct hist_data	he_tx;
	timestamp_t		he_last_update;
	int			he_index;
};

struct history
{
	struct hist_elem	h_read;
	struct hist_elem	h_sec;
	struct hist_elem	h_min;
	struct hist_elem	h_hour;
	struct hist_elem	h_day;
};

struct rate
{
	b_cnt_t			r_total_sum;
	b_cnt_t			r_total_prev;
	b_cnt_t			r_current;
	b_cnt_t			r_prev;
	rate_cnt_t		r_tps;
	timestamp_t		r_last_update;
};

#define RX_PROVIDED			0x01
#define TX_PROVIDED			0x02
#define IS64BIT				0x04

#define ATTR_FLAG_RX_ENABLED		0x01
#define ATTR_FLAG_TX_ENABLED		0x02
#define ATTR_FLAG_HISTORY		0x04
#define ATTR_FLAG_IGNORE_OVERFLOWS	0x08
#define ATTR_FLAG_IS_RATE		0x10
#define ATTR_FLAG_IS_64BIT		0x20
#define ATTR_FLAG_IS_USAGE		0x40

struct stat_attr_s;

#define STAT_ATTR_COMMON				\
	uint16_t		a_type;			\
	uint8_t			a_flags;		\
	struct unit *		a_unit;			\
	struct stat_attr *	a_next;			\
	timestamp_t		a_last_distribution;	\
	timestamp_t		a_updated;		\
	struct rate		a_rx;			\
	struct rate		a_tx

struct stat_attr
{
	STAT_ATTR_COMMON;
};

struct stat_attr_hist
{
	STAT_ATTR_COMMON;
	struct history		a_hist;
};

struct node;

struct item
{
	char            i_name[IFNAME_MAX];
	char *		i_desc;
	uint32_t        i_handle;
	struct node *		i_node;
	struct item_group *	i_group;
	struct item *		i_parent;
	struct item *		i_childs;
	struct item *		i_childs_tail;
	uint16_t        i_level;
	uint16_t        i_nattrs; 
	uint16_t		i_lifetime;
	struct stat_attr *	i_attr_sel[MAX_GRAPHS];
	struct stat_attr *	i_attrs[ATTR_HASH_MAX];
	uint8_t		i_unit[MAX_GRAPHS];
	uint16_t	i_graph_sel;
	uint32_t	i_flags;
	int		i_rx_usage;
	int		i_tx_usage;
	int		i_major_attr;
	int		i_minor_attr;
	int		i_usage_attr;
	struct it_item *	i_tab;

	struct item *		i_next;
	struct item *		i_prev;

};

#define ITEM_FLAG_FOLDED	0x01
#define ITEM_FLAG_UPDATED	0x02
#define ITEM_FLAG_EXCLUDE	0x04
#define ITEM_FLAG_LOCAL		0x08

static inline int item_folded(struct item *item)
{
	return item->i_flags & ITEM_FLAG_FOLDED;
}

extern struct item * lookup_item(struct node *, struct item_group *,
				 const char *, uint32_t, struct item *);
extern struct item * search_item(struct item_group *, const char *, uint32_t,
				 struct item *);
extern void foreach_child(struct item_group *, struct item *,
			  void (*cb)(struct item *, void *), void *);
extern void notify_update(struct item *, timestamp_t *);
extern void increase_lifetime(struct item *, int);
extern void reset_item(struct item *);
extern void remove_unused_items(struct item *);

extern void update_attr(struct item *, int, b_cnt_t, b_cnt_t, int);
extern void restore_attr(struct item *, int, b_cnt_t, b_cnt_t);
extern void item_parse_policy(const char *);
extern void parse_attr_policy(const char *);
extern void foreach_attr(struct item *, void (*cb)(struct stat_attr *, void *), void *);

extern struct stat_attr *lookup_attr(struct item *, int);
extern b_cnt_t attr_get_rx(struct stat_attr *);
extern rate_cnt_t attr_get_rx_rate(struct stat_attr *);
extern inline b_cnt_t attr_get_rx_last(struct stat_attr *);
extern b_cnt_t attr_get_tx(struct stat_attr *);
extern rate_cnt_t attr_get_tx_rate(struct stat_attr *);
extern inline b_cnt_t attr_get_tx_last(struct stat_attr *);

extern int next_attr(void);
extern int set_graph_unit(int unit);


extern struct stat_attr *current_attr(struct item *item, int graph);

#endif
